@@ -7,14 +7,14 @@ from typing import Union
 import pandas as pd
 import structlog
 from openstf_dbc.services.prediction_job import PredictionJobDataClass
-from sklearn.base import RegressorMixin
 
 from openstf.feature_engineering.feature_applicator import (
     OperationalPredictFeatureApplicator,
 )
 from openstf.model.confidence_interval_applicator import ConfidenceIntervalApplicator
 from openstf.model.fallback import generate_fallback
-from openstf.model.serializer import PersistentStorageSerializer
+from openstf.model.regressors.regressor import OpenstfRegressor
+from openstf.model.serializer import MLflowSerializer
 from openstf.pipeline.utils import generate_forecast_datetime_range
 from openstf.postprocessing.postprocessing import (
     add_prediction_job_properties_to_forecast,
@@ -23,7 +23,7 @@ from openstf.validation import validation
 
 
 def create_forecast_pipeline(
-    pj: Union[dict, PredictionJobDataClass],
+    pj: PredictionJobDataClass,
     input_data: pd.DataFrame,
     trained_models_folder: Union[str, Path],
 ) -> pd.DataFrame:
@@ -35,7 +35,7 @@ def create_forecast_pipeline(
     Expected prediction job keys: "id",
 
     Args:
-        pj (Union[dict, PredictionJobDataClass]): Prediction job
+        pj (PredictionJobDataClass): Prediction job
         input_data (pd.DataFrame): Training input data (without features)
         trained_models_folder (Path): Path where trained models are stored
 
@@ -45,17 +45,17 @@ def create_forecast_pipeline(
 
     """
     # Load most recent model for the given pid
-    model = PersistentStorageSerializer(
+    model, modelspecs = MLflowSerializer(
         trained_models_folder=trained_models_folder
-    ).load_model(pid=pj["id"])
+    ).load_model(pj["id"])
 
     return create_forecast_pipeline_core(pj, input_data, model)
 
 
 def create_forecast_pipeline_core(
-    pj: Union[dict, PredictionJobDataClass],
+    pj: PredictionJobDataClass,
     input_data: pd.DataFrame,
-    model: RegressorMixin,
+    model: OpenstfRegressor,
 ) -> pd.DataFrame:
     """Create forecast pipeline (core)
 
@@ -63,12 +63,12 @@ def create_forecast_pipeline_core(
     This pipeline has no database or persisitent storage dependencies.
 
     Expected prediction job keys: "resolution_minutes", "horizon_minutes", "id", "type",
-        "name", "model_type_group", "quantiles"
+        "name", "quantiles"
 
     Args:
-        pj (Union[dict, PredictionJobDataClass]): Prediction job.
+        pj (PredictionJobDataClass): Prediction job.
         input_data (pandas.DataFrame): Iput data for the prediction.
-        model (RegressorMixin): Model to use for this prediction.
+        model (OpenstfRegressor): Model to use for this prediction.
 
     Returns:
         forecast (pandas.DataFrame)
@@ -84,14 +84,12 @@ def create_forecast_pipeline_core(
     data_with_features = OperationalPredictFeatureApplicator(
         # TODO use saved feature_names (should be saved while training the model)
         horizons=[0.25],
-        feature_names=model._Booster.feature_names,
+        feature_names=model.feature_names,
     ).add_features(validated_data)
 
     # Prep forecast input by selecting only the forecast datetime interval (this is much smaller than the input range)
     # Also drop the load column
-    forecast_start, forecast_end = generate_forecast_datetime_range(
-        pj["resolution_minutes"], pj["horizon_minutes"]
-    )
+    forecast_start, forecast_end = generate_forecast_datetime_range(data_with_features)
     forecast_input_data = data_with_features[forecast_start:forecast_end].drop(
         columns="load"
     )
