@@ -9,6 +9,7 @@ import pandas as pd
 import structlog
 from openstf_dbc.services.prediction_job import PredictionJobDataClass
 
+from openstf.data_classes.model_specifications import ModelSpecificationDataClass
 from openstf.exceptions import (
     InputDataInsufficientError,
     InputDataWrongColumnOrderError,
@@ -18,7 +19,7 @@ from openstf.feature_engineering.feature_applicator import TrainFeatureApplicato
 from openstf.metrics.reporter import Report, Reporter
 from openstf.model.model_creator import ModelCreator
 from openstf.model.regressors.regressor import OpenstfRegressor
-from openstf.model.serializer import PersistentStorageSerializer
+from openstf.model.serializer import MLflowSerializer
 from openstf.model.standard_deviation_generator import StandardDeviationGenerator
 from openstf.model_selection.model_selection import split_data_train_validation_test
 from openstf.validation import validation
@@ -31,7 +32,7 @@ PENALTY_FACTOR_OLD_MODEL: float = 1.2
 
 
 def train_model_pipeline(
-    pj: Union[dict, PredictionJobDataClass],
+    pj: PredictionJobDataClass,
     input_data: pd.DataFrame,
     check_old_model_age: bool,
     trained_models_folder: Union[str, Path],
@@ -42,7 +43,7 @@ def train_model_pipeline(
         "feature_names"
 
     Args:
-        pj (Union[dict, PredictionJobDataClass]): Prediction job
+        pj (PredictionJobDataClass): Prediction job
         input_data (pd.DataFrame): Raw training input data
         check_old_model_age (bool): Check if training should be skipped because the model is too young
         trained_models_folder (Path): Path where trained models are stored
@@ -53,17 +54,19 @@ def train_model_pipeline(
     """
     # Intitialize logger and serializer
     logger = structlog.get_logger(__name__)
-    serializer = PersistentStorageSerializer(trained_models_folder)
+    serializer = MLflowSerializer(trained_models_folder)
 
     # Get old model and age
     try:
-        old_model = serializer.load_model(pid=pj["id"])
+        old_model, modelspecs = serializer.load_model(pj["id"])
         old_model_age = (
             old_model.age
         )  # Age attribute is openstf specific and is added by the serializer
-    except FileNotFoundError:
+    except (AttributeError, FileNotFoundError):
         old_model = None
         old_model_age = float("inf")
+        # create basic modelspecs
+        modelspecs = ModelSpecificationDataClass(id=pj["id"])
         logger.warning("No old model found, training new model", pid=pj["id"])
 
     # Check old model age and continue yes/no
@@ -75,7 +78,9 @@ def train_model_pipeline(
 
     # Train model with core pipeline
     try:
-        model, report = train_model_pipeline_core(pj, input_data, old_model)
+        model, report, modelspecs_updated = train_model_pipeline_core(
+            pj, modelspecs, input_data, old_model
+        )
     except OldModelHigherScoreError as OMHSE:
         logger.error("Old model is better than new model", pid=pj["id"], exc_info=OMHSE)
         return
@@ -97,33 +102,39 @@ def train_model_pipeline(
         raise InputDataWrongColumnOrderError(IDWCOE)
 
     # Save model
-    serializer.save_model(model, pj=pj, report=report)
+    serializer.save_model(model, pj=pj, modelspecs=modelspecs_updated, report=report)
+
+    # Clean up older models
+    serializer.remove_old_models(pj=pj)
 
 
 def train_model_pipeline_core(
-    pj: Union[dict, PredictionJobDataClass],
+    pj: PredictionJobDataClass,
+    modelspecs: ModelSpecificationDataClass,
     input_data: pd.DataFrame,
     old_model: OpenstfRegressor = None,
     horizons: List[float] = None,
-) -> Tuple[OpenstfRegressor, Report]:
+) -> Tuple[OpenstfRegressor, Report, ModelSpecificationDataClass]:
     """Train model core pipeline.
-    Trains a new model given a predction job, input data and compares it to an old model.
-    This pipeline has no database or persisitent storage dependencies.
+    Trains a new model given a prediction job, input data and compares it to an old model.
+    This pipeline has no database or persistent storage dependencies.
 
     TODO once we have a data model for a prediction job this explantion is not
     required anymore.
 
-    For training a model the following keys in the prediction job dictionairy are
+    For training a model the following keys in the prediction job are
     expected:
         "id"            Only used for logging
         "model"         Model type, any of "xgb", "lgb",
+    And in modelspecs:
         "hyper_params"  Hyper parameters dictionairy specific to the model_type
         "feature_names"      List of features to train model on or None to use all features
 
     Args:
-        pj (Union[dict, PredictionJobDataClass]): Prediction job
+        pj (PredictionJobDataClass): Prediction job
+        modelspecs (ModelSpecificationDataClass): Dataclass containing model specifications
         input_data (pd.DataFrame): Input data
-        old_model (RegressorMixin, optional): Old model to compare to. Defaults to None.
+        old_model (OpenstfRegressor, optional): Old model to compare to. Defaults to None.
         horizons (List[float]): horizons to train on in hours.
 
     Raises:
@@ -132,7 +143,7 @@ def train_model_pipeline_core(
         OldModelHigherScoreError: When old model is better than new model.
 
     Returns:
-        Tuple[RegressorMixin, Report]: Trained model and report (with figures)
+        Tuple[OpenstfRegressor, Report, ModelSpecificationDataClass]: Trained model and report (with figures)
     """
 
     if horizons is None:
@@ -142,9 +153,9 @@ def train_model_pipeline_core(
 
     # Call common pipeline
     model, report, train_data, validation_data, test_data = train_pipeline_common(
-        pj, input_data, horizons
+        pj, modelspecs, input_data, horizons
     )
-
+    modelspecs.feature_names = list(train_data.columns)
     logging.info("Fitted a new model, not yet stored")
 
     # Check if new model is better than old model
@@ -176,22 +187,29 @@ def train_model_pipeline_core(
         except ValueError as e:
             logger.info("Could not compare to old model", pid=pj["id"], exc_info=e)
 
-    return model, report
+    return model, report, modelspecs
 
 
 def train_pipeline_common(
-    pj: Union[dict, PredictionJobDataClass],
+    pj: PredictionJobDataClass,
+    modelspecs: ModelSpecificationDataClass,
     input_data: pd.DataFrame,
     horizons: List[float],
     test_fraction: float = 0.0,
     backtest: bool = False,
+    test_data_predefined: pd.DataFrame = pd.DataFrame(),
 ) -> Tuple[OpenstfRegressor, pd.DataFrame, pd.DataFrame, pd.DataFrame]:
     """Common pipeline shared with operational training and backtest training
 
     Args:
-        pj (Union[dict, PredictionJobDataClass]): Prediction job
+        pj (PredictionJobDataClass): Prediction job
+        modelspecs (ModelSpecificationDataClass): Dataclass containing model specifications
         input_data (pd.DataFrame): Input data
         horizons (List[float]): horizons to train on in hours.
+        test_fraction (float): fraction of data to use for testing
+        backtest (bool): boolean if we need to do a backtest
+        test_data_predefined (pd.DataFrame): Predefined test data frame to be used in the pipeline
+            (empty data frame by default)
 
     Returns:
         Tuple[RegressorMixin, Report, pd.DataFrame, pd.DataFrame, pd.DataFrame]: Trained model, report
@@ -209,9 +227,8 @@ def train_pipeline_common(
             "Missing the load column in the input dataframe"
         )
 
-        # Validate and clean data
+    # Validate and clean data
     validated_data = validation.clean(validation.validate(pj["id"], input_data))
-
     # Check if sufficient data is left after cleaning
     if not validation.is_data_sufficient(validated_data):
         raise InputDataInsufficientError(
@@ -219,9 +236,18 @@ def train_pipeline_common(
         )
 
     data_with_features = TrainFeatureApplicator(
-        horizons=horizons, feature_names=pj["feature_names"]
+        horizons=horizons, feature_names=modelspecs.feature_names
     ).add_features(validated_data)
 
+    # if test_data is predefined, apply the pipeline only on the remaining data
+    if not test_data_predefined.empty:
+        test_data_predefined = data_with_features[
+            data_with_features.index.isin(test_data_predefined.index)
+        ].sort_index()
+        data_with_features = data_with_features[
+            ~data_with_features.index.isin(test_data_predefined.index)
+        ].sort_index()
+
     # Split data
     (
         peaks,
@@ -235,6 +261,10 @@ def train_pipeline_common(
         back_test=backtest,
     )
 
+    # if test_data is predefined, use this over the returned test_data of split function
+    if not test_data_predefined.empty:
+        test_data = test_data_predefined
+
     # Test if first column is "load" and last column is "horizon"
     if train_data.columns[0] != "load" or train_data.columns[-1] != "horizon":
         raise InputDataWrongColumnOrderError(
@@ -258,7 +288,7 @@ def train_pipeline_common(
     # Set relevant hyperparameters
     valid_hyper_parameters = {
         key: value
-        for key, value in pj["hyper_params"].items()
+        for key, value in modelspecs.hyper_params.items()
         if key in model.get_params().keys()
     }
 
@@ -271,7 +301,7 @@ def train_pipeline_common(
         verbose=False,
     )
     # Gets the feature importance df or None if we don't have feature importance
-    model.feature_importance_dataframe = model.set_feature_importance(train_x.columns)
+    model.feature_importance_dataframe = model.set_feature_importance()
 
     logging.info("Fitted a new model, not yet stored")
 
@@ -285,17 +315,3 @@ def train_pipeline_common(
     report = reporter.generate_report(model)
 
     return model, report, train_data, validation_data, test_data
-
-
-def get_model_age(trained_models_folder: str, pid: int) -> float:
-    """returns age of most recently trained model in days.
-    If no previous model can be found, this returns float(inf).
-
-    Args:
-        trained_models_folder: str
-        pid: int
-
-    Returns:
-        float: age of most recent model in days"""
-    serializer = PersistentStorageSerializer(trained_models_folder)
-    return serializer.determine_model_age_from_pid(pid)
